#ifndef UTILS_HPP
#define UTILS_HPP

#include "Arduino.h"
#include "WString.h"

namespace fw
{
    typedef enum firewall_targets : uint8_t
    {
        TARGET_DROP = 1,
        TARGET_ACCEPT = 2,
    } firewall_target_t;

    typedef enum firewall_protocols : uint8_t
    {
        PROTOCOL_TCP = 6,
        PROTOCOL_UDP = 17,
        PROTOCOL_ALL = 255,
    } firewall_protocol_t;

    typedef enum ok : uint8_t
    {
        SUCCESS = 0,
        ERROR = 1,
        NO_ACTION = 2,
    } ok_t;

    typedef enum auth : uint8_t
    {
        AUTHENTICATED = 0,
        DENIED = 1,
    } auth_t;

    static const uint8_t IPV4ADDRESS_LENGTH = 16;

    /**
     * @brief the typedef to describe a Firewall rule
     *
     */
    typedef struct firewall_rules
    {
        uint8_t key;
        char ip[IPV4ADDRESS_LENGTH];
        uint16_t port_from;
        uint16_t port_to;
        firewall_protocol_t protocol;
        firewall_target_t target;
        struct firewall_rules *next;
    } firewall_rule_t;

    /**
     * @brief the typedef to describe a network packet
     *
     */
    typedef struct my_packet
    {
        char ip[IPV4ADDRESS_LENGTH];
        firewall_protocol_t protocol;
        uint16_t port;
    } my_packet_t;

    static const uint8_t firewall_fields_amount = 6;
    const char firewall_fields[firewall_fields_amount][10] = {"key", "ip", "port_from", "port_to", "protocol", "target"};
    typedef enum firewall_fields : uint8_t
    {
        KEY,
        IP,
        PORT_FROM,
        PORT_TO,
        PROTOCOL,
        TARGET,
    } firewall_fields_t;

    static const uint8_t CREDENTIALS_LENGTH = 32;

    /**
     * @brief the typedef to describe credentials of the API
     *
     */
    typedef struct credentials
    {
        char password[CREDENTIALS_LENGTH];
        char username[CREDENTIALS_LENGTH];
    } credential_t;

    /**
     * @brief the typedef to describe an API endpoint
     *
     */
    typedef struct api_endpoints
    {
        char uri[60];
        char method[7];
        char description[30];
        struct api_endpoints *next;
    } api_endpoint_t;

    /**
     * @brief converting protocol type to string
     *
     * @param protocol
     * @return String
     */
    String protocol_to_string(firewall_protocol_t &protocol);

    /**
     * @brief converting string to protocol type
     *
     * @param protocol
     * @return firewall_protocol_t
     */
    firewall_protocol_t string_to_protocol(String &protocol);

    /**
     * @brief converting target type to string
     *
     * @param target
     * @return String
     */
    String target_to_string(firewall_target_t &target);

    /**
     * @brief converting string to target type
     *
     * @param target
     * @return firewall_target_t
     */
    firewall_target_t string_to_target(String &target);

    /**
     * @brief running an endless loop to prevent further damage
     *
     */
    void endless_loop();

    /**
     * @brief returns if number is between or equal to lower and upper
     *
     * @param number
     * @param lower
     * @param upper
     * @return true
     * @return false
     */
    bool is_in_range(const uint16_t number, const uint16_t lower, const uint16_t upper);
}

#endif
